// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of whiteboard_core;

class WhiteBoardContainerOptions {
  const WhiteBoardContainerOptions.defaultOptions({this.bottomLeft})
      : bottomRight = const BottomRight(WhiteboardItems.defaultBottomRight),
        topRight = const TopRight(WhiteboardItems.defaultTopRight),
        topLeft = const TopLeft(WhiteboardItems.defaultTopLeft);

  final BottomRight? bottomRight;
  final TopRight? topRight;
  final TopLeft? topLeft;
  final BottomLeft? bottomLeft;

  WhiteBoardContainerOptions({
    this.bottomRight,
    this.topRight,
    this.topLeft,
    this.bottomLeft,
  });

  @override
  String toString() {
    return 'WhiteBoardContainerOptions{bottomRight: $bottomRight, topRight: $topRight, topLeft: $topLeft}';
  }
}


/// [Items]
class WhiteboardItems {

  static const defaultBottomRight = [selectItem, penItem, shapeItem,defaultBottomRightMoreItem];
  static const defaultTopRight = [defaultTopRightMoreItem];
  static const defaultTopLeft = [pageBoardInfoItem,previewItem];

  static const selectItem = Items(
    tool: 'select',
    hint: '选择',
  );

  static const penItem = Items(
    tool: 'pen',
    hint: '画笔',
    stack: 'horizontal',
  );

  static const shapeItem = Items(
    tool: 'shape',
    hint: '图形',
    stack: 'horizontal',
  );

  static const pageBoardInfoItem = Items(
    tool: 'pageBoardInfo',
  );

  static const previewItem = Items(
    tool: 'preview',
    hint: '预览',
    previewSliderPosition: 'right',
  );

  static const defaultBottomRightMoreItem = Items(
    tool: 'multiInOne',
    hint: '更多',
    subItems: defaultBottomRightMoreSubItems,
  );

  static const defaultBottomRightMoreSubItems = [
    elementEraserSubItem,
    clearSubItem,
    undoSubItem,
    redoSubItem,
  ];

  static const defaultTopRightMoreItem = Items(
    tool: 'multiInOne',
    hint: '更多',
    subItems: defaultTopRightSubItems,
  );

  static const defaultTopRightSubItems = [
    fitToContentSubItem,
    fitToDocSubItem,
    panSubItem,
    zoomInSubItem,
    zoomOutSubItem,
    visionLockSubItem,
    zoomLevelSubItem,
  ];
}

/// [WhiteboardItems.defaultTopRight]
const fitToContentSubItem = SubItems(tool: 'fitToContent');

const fitToDocSubItem = SubItems(tool: 'fitToDoc');
const panSubItem = SubItems(tool: 'pan');
const zoomInSubItem = SubItems(tool: 'zoomIn');
const zoomOutSubItem = SubItems(tool: 'zoomOut');
const visionLockSubItem = SubItems(tool: 'visionLock');
const zoomLevelSubItem = SubItems(tool: 'zoomLevel');

/// [WhiteboardItems.defaultBottomRightMoreSubItems]
const elementEraserSubItem = SubItems(tool: 'element-eraser');

const clearSubItem = SubItems(tool: 'clear');

const undoSubItem = SubItems(tool: 'undo');

const redoSubItem = SubItems(tool: 'redo');

/// 支持[BottomRight],[TopRight],[TopLeft]自定义
abstract class _PositionContainerOptions {
  final List<Items?>? items;

  final String? position;

  const _PositionContainerOptions({required this.items ,required this.position});

  Map<String, dynamic> toJson() => {
    if (position != null) 'position': position,
    if (items != null) 'items':  items!.map((e) => e?.toJson()).toList(),
  };

  @override
  String toString() {
    return '_PositionContainerOptions{items: $items}';
  }
}

class BottomRight extends _PositionContainerOptions {
  const BottomRight(List<Items?>? items,{String? position = 'bottomRight'})
      : super(items: items, position: position);
}

class BottomLeft extends _PositionContainerOptions {
  const BottomLeft(List<Items?> items, {String? position = 'bottomLeft'})
      : super(items: items, position: position);
}

class TopRight extends _PositionContainerOptions {
  const TopRight(List<Items?> items, {String? position = 'topRight'})
      : super(items: items, position: position);
}

class TopLeft extends _PositionContainerOptions {
  const TopLeft(List<Items?> items, {String? position = 'topLeft'})
      : super(items: items, position: position);
}