// Copyright (c) 2014-2020 NetEase, Inc.
// All right reserved.

part of whiteboard_core;

class _RoomWebWhiteBoardImp with NEWhiteBoard {
  factory _RoomWebWhiteBoardImp() => _getInstance();

  static _RoomWebWhiteBoardImp get instance => _getInstance();

  static _RoomWebWhiteBoardImp? _instance;

  Future<WebViewController>? _controller;

  // WebViewController? get controller => _controller;
  //
  // set controller(WebViewController? value) {
  //   _controller = value;
  // }

  _RoomWebWhiteBoardImp._internal();

  static _RoomWebWhiteBoardImp _getInstance() {
    _instance ??= _RoomWebWhiteBoardImp._internal();
    return _instance!;
  }

  ///是否能开启白板编辑工具栏
  bool _enableWhiteBoardDraw = true;

  bool get enableWhiteBoardDraw => _enableWhiteBoardDraw;

  ///是否编辑状态
  bool _isEditing = false;

  ///是否已经释放了
  bool _isRelease = false;

  ///是否已经登录
  bool _isLogin = false;

  ///白板地址
  String? _whiteBoardUrl;

  bool get isEditing => _isEditing;

  set isEditing(bool value) {
    _isEditing = value;
  }

  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///
  ///  ///  ///  jsCallFlutter start
  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///
  @override
  Future<String?> login(
    RoomWBLoginInfo whiteBoardUserInfo,
  ) async {
    _whiteBoardUrl = whiteBoardUserInfo.url;
    String? result;
    try {
      result = await evaluateJavascript(WhiteBoardWebAction.jsLoginIMAndJoinWB,
          params: whiteBoardUserInfo);
    } catch (e) {
      return e.toString();
    }
    return result;
  }

  @override
  Future<String?> setContainerOptions(
      {WhiteBoardContainerOptions whiteBoardContainerOptions =
          const WhiteBoardContainerOptions.defaultOptions()}) async {
    var result = await evaluateJavascriptJsDirectCallSetContainerOptions(
        whiteBoardContainerOptions);
    return result;
  }

  Future<String?> leaveWB() async {
    String? result;
    try {
      result = await evaluateJavascript(WhiteBoardWebAction.webLeaveWB);
    } catch (e) {
      return e.toString();
    }
    return Future.value(result);
  }


  Future<String?> _jsEnableDraw(bool enable) async {
    String? result;
    try {
      var params = RoomWBEnableDraw(enable: enable);
      result = await evaluateJavascriptJsDirectCallEnable(
          WhiteBoardWebAction.enableDraw,
          enable: params.enable);
    } catch (e) {
      return e.toString();
    }
    return result;
  }

  /// 兼容菜单默认显示的API接口逻辑
  Future<String?> evaluateJavascriptJsDirectCallToolCollection(
      bool visible) async {
    // var exclude =
    //     !visible ? '' : '"fitToContentDoc", "zoomOut", "zoomLevel", "zoomIn"';
    var params =
        '{"topLeft": {"visible": $visible},"topRight": {"visible": true},"bottomRight": {"visible": $visible}}';
    return await _controller?.then((controller) => evaluateJavascriptWithDirect(
        controller,
        target: 'toolCollection',
        action: 'setVisibility',
        params: params));
  }

  // /// 兼容菜单默认显示的API接口逻辑
  Future<String?> evaluateJavascriptJsDirectCallSetContainerOptions(
      WhiteBoardContainerOptions options) async {
    var params = [options.bottomRight, options.topRight, options.topLeft];
    if (options.bottomLeft != null) params.add(options.bottomLeft);
    // var params =
    //     '[{"position":"bottomRight","items":[{"tool":"select","hint":"选择"},{"tool":"pen","hint":"画笔","stack":"horizontal"},{"tool":"shape","hint":"图形","stack":"horizontal"},{"tool":"multiInOne","hint":"更多","subItems":[{"tool":"element-eraser"},{"tool":"clear"},{"tool":"undo"},{"tool":"redo"}]}]},{"position":"topRight","items":[{"tool":"multiInOne","hint":"更多","subItems":[{"tool":"fitToContent"},{"tool":"fitToDoc"},{"tool":"pan"},{"tool":"zoomIn"},{"tool":"zoomOut"},{"tool":"visionLock"}]},{"tool":"zoomLevel"}]},{"position":"topLeft","items":[{"tool":"pageBoardInfo"},{"tool":"preview","hint":"预览","previewSliderPosition":"right"}]}]';
    return await _controller?.then((controller) => evaluateJavascriptWithDirect(
        controller,
        target: 'toolCollection',
        action: 'setContainerOptions',
        params: json.encode(params)));
  }

  Future<String?> evaluateJavascriptJsDirectCallEnable(String action,
      {required bool enable}) async {
    ///兼容接口，默认关闭工具栏和可编辑属性
    await evaluateJavascriptJsDirectCallToolCollection(enable);
    return _controller?.then((controller) => evaluateJavascriptWithDirect(
        controller,
        target: 'drawPlugin',
        action: action,
        params: enable));
  }

  /// http://dev.yunxin.163.com/docs/product/%E4%BA%92%E5%8A%A8%E7%99%BD%E6%9D%BF/%E5%BF%AB%E9%80%9F%E5%85%A5%E9%97%A8/Javascript%E6%8E%A5%E5%85%A5
  Future<String?> evaluateJavascriptWithDirect(WebViewController controller,
      {required String target, required String action, Object? params}) async {
    return await controller.evaluateJavascript(
        'javascript:WebJSBridge({"action":"jsDirectCall","param":{"target":"$target","action":"$action","params":[${params}]}})');
  }

  Future<String?> evaluateJavascript(String action, {Object? params}) async {
    return await _controller?.then((controller) => controller.evaluateJavascript(
        'javascript:WebJSBridge({"action":"$action","param":${jsonEncode(params ?? '{}')}})'));
  }

  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///
  ///  ///  ///  jsCallFlutter end
  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///  ///

  @override
  Future<int> enableDraw(bool enable) async {
    if (_enableWhiteBoardDraw != enable && !_isRelease) {
      _enableWhiteBoardDraw = enable;
      isEditing = enable;
      await _jsEnableDraw(enable);
    }
    return Future.value(_isRelease ? -1 : 0);
  }

  @override
  Future<int> release() {
    _isRelease = true;
    _enableWhiteBoardDraw = true;
    _isEditing = false;
    _isLogin = false;
    _controller = null;
    return Future.value(0);
  }

  @override
  bool getEditWhiteBoardStatus() {
    return _isEditing;
  }

  @override
  String getWhiteboardUrl() => _whiteBoardUrl ?? defaultWhiteBoardUrl;

  @override
  bool isWhiteboardLogin() => _isLogin;

  @override
  JavascriptChannel initWhiteboardJavascriptChannel(
    Future<WebViewController> javascriptController, {
    NEWhiteboardPageLoaded? onWhiteboardPageLoaded,
    NEWhiteboardJoinWBSucceed? onWhiteboardJoinWBSucceed,
    NEWhiteboardErrorCallback? onWhiteboardErrorCallback,
    NEWhiteboardLeave? onWhiteboardLeave,
    NEWhiteboardInterceptLog? onWhiteboardInterceptLog,
  }) {
    _isRelease = false;
    _controller = javascriptController;
    return JavascriptChannel(
        name: javascriptChannelName,
        onMessageReceived: (JavascriptMessage message) {
          handleJSMessage(
            message,
            onWhiteboardPageLoaded,
            onWhiteboardJoinWBSucceed,
            onWhiteboardErrorCallback,
            onWhiteboardLeave,
            onWhiteboardInterceptLog,
          );
        });
  }

  void handleJSMessage(
    JavascriptMessage message,
    NEWhiteboardPageLoaded? onWhiteboardPageLoaded,
    NEWhiteboardJoinWBSucceed? onWhiteboardJoinWBSucceed,
    NEWhiteboardErrorCallback? onWhiteboardErrorCallback,
    NEWhiteboardLeave? onWhiteboardLeave,
    NEWhiteboardInterceptLog? onWhiteboardInterceptLog,
  ) {
    var json = jsonDecode(message.message);
    var dataFromJs = RoomWebWBBase.fromJson(json as Map<String, dynamic>);
    if (onWhiteboardInterceptLog != null){
      onWhiteboardInterceptLog.call(json);
    }
    switch (dataFromJs.action) {
      case 'webPageLoaded':
        if(onWhiteboardPageLoaded != null) onWhiteboardPageLoaded.call();
        break;
      case 'webJoinWBSucceed':
        _isLogin = true;
        if(onWhiteboardJoinWBSucceed != null) onWhiteboardJoinWBSucceed.call();
        break;
      case 'webLoginIMFailed':
      case 'webError':
      case 'webJoinWBFailed':
      case 'webCreateWBFailed':
        _isLogin = false;
        var code = (dataFromJs.param['code'] ?? -1) as int;
        var msg = (dataFromJs.param['msg'] ?? '') as String;
        if(onWhiteboardErrorCallback != null){
          onWhiteboardErrorCallback.call(code:code,msg:msg);
        }
        break;
      case 'webLeaveWB':
        _isLogin = false;
        if (onWhiteboardLeave != null) onWhiteboardLeave.call();
        break;
    }
  }
}
